#include <utility>
#include <memory>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <afx.h>
using namespace std;

template <class Container_Type, 
          class Elem_Type, 
          class Allocator = allocator<Elem_Type> >


//define an iterator 
class MFCArrayIter : public _Ranit<Elem_Type, Allocator::difference_type> {

    typedef Elem_Type & reference;
    typedef    Allocator::difference_type difference_type; 
    
public:
    MFCArrayIter(Container_Type * pContainer = 0)
    : m_pContainer(pContainer), m_Ptr(0){}

    MFCArrayIter(Container_Type * pContainer, int offset)
    : m_pContainer(pContainer), m_Ptr(offset){}
        
    MFCArrayIter(const MFCArrayIter & other)
    : m_pContainer(other.m_pContainer), m_Ptr(other.m_Ptr){}

    MFCArrayIter begin(){
        return MFCArrayIter(m_pContainer, 0);
        }

    MFCArrayIter end(){
        return MFCArrayIter(m_pContainer, m_pContainer->GetSize());
        }

    reference operator*() const    { 
        return (*m_pContainer)[m_Ptr]; 
        }
        
    reference operator->() const    { 
        return (*m_pContainer)[m_Ptr]; 
        }
        
    MFCArrayIter& operator++(){
        _Inc();
        return (*this); 
        }
        
    MFCArrayIter operator++(int){
        MFCArrayIter _Tmp = *this;
        _Inc();
        return (_Tmp); 
        }
        
    MFCArrayIter& operator--(){
        _Dec();
        return (*this); 
        }
        
    MFCArrayIter  operator--(int){
        MFCArrayIter _Tmp = *this;
        _Dec();
        return (_Tmp); 
        }
        
    MFCArrayIter& operator+=(difference_type _N){
        m_Ptr += _N;
        m_Ptr %= m_pContainer->GetSize();
        return (*this); 
        }
        
    MFCArrayIter& operator-=(difference_type _N){
        return (*this += -_N); 
        }
        
    MFCArrayIter operator+(difference_type _N) const{
        MFCArrayIter _Tmp = *this;
        return (_Tmp += _N); 
        }
        
    MFCArrayIter operator-(difference_type _N) const{
        MFCArrayIter _Tmp = *this;
        return (_Tmp -= _N); 
        }
        
    difference_type operator-(const MFCArrayIter & _X) const{
        return m_Ptr - _X.m_Ptr;
        }
        
    reference operator[](difference_type _N) const{
        return (*(*this + _N)); 
        }
        
    bool operator==(const MFCArrayIter& _X) const{
        return (m_Ptr == _X.m_Ptr && (m_pContainer)
                == (_X.m_pContainer)); 
        }
        
    bool operator!=(const MFCArrayIter& _X) const{
        return (!(*this == _X));
        }
        
    bool operator<(const MFCArrayIter& _X) const{
        return (m_Ptr < _X.m_Ptr && (m_pContainer) == (_X.m_pContainer));
        }
        
    bool operator>(const MFCArrayIter& _X) const{
        return (_X < *this); 
        }
        
    bool operator<=(const MFCArrayIter& _X) const{
        return (!(_X < *this)); 
        }
        
    bool operator>=(const MFCArrayIter& _X) const{
        return (!(*this < _X)); 
        }
protected:
    //the _Dec and _Inc functions wrap around
    void _Dec(){
        if (m_Ptr != 0)
            --m_Ptr;
        else
            m_Ptr = m_pContainer->GetSize() - 1; 
        }
        
    void _Inc(){
        if (m_Ptr < m_pContainer->GetSize())
            ++m_Ptr;
        else
            m_Ptr = 0; 
        }
        
    Container_Type*    m_pContainer;
    int                m_Ptr;
};


template<class Container_Type, class Value_Type>
class MFC_back_insert_iterator
    : public iterator<output_iterator_tag, void, void> {

public:
    explicit MFC_back_insert_iterator(Container_Type& _X)
        : container(_X) {}
    
    MFC_back_insert_iterator & operator=(const Value_Type& _V){
        container.Add(_V);
        return (*this); 
    }
    
    MFC_back_insert_iterator & operator*(){
        return (*this); 
    }
    
    MFC_back_insert_iterator & operator++(){
        return (*this); 
    }
    
    MFC_back_insert_iterator & operator++(int){
        return (*this); 
    }

protected:
    Container_Type& container;
    
};



