// TEST.CPP - Demonstration application for lock hierarchy
//
// Author: Jeff Claar

#define STRICT
#include <windows.h>
#include <stdio.h>
#include "cs.hpp"

// Define BAD_LOCK to acquire locks out of order.
//#define BAD_LOCK

void main()
{
    // Create some synchronization objects with
    // lock levels 1 through 4.
    C_CSLockLevel cs1(1), cs2(2);
    C_MutexLockLevel mutex1(3), mutex2(4);

    // Acquire the objects in the proper order.
    printf("Acquiring lock with level 1.\n");
    cs1.Enter();
    printf("Acquiring lock with level 2.\n");
    cs2.Enter();

    // Use C_MutexLockLevel::EnterMultiple for mutexes.
    C_MutexLockLevel* mutexes[] = {
        &mutex1,
        &mutex2
    };
    printf("Acquiring locks with levels 3 and 4.\n");
    C_MutexLockLevel::EnterMultiple(2, mutexes, TRUE, INFINITE);

    // Unlock them. The order does not matter
    // when unlocking objects.
    cs2.Leave();
    mutex2.Leave();
    mutex1.Leave();
    cs1.Leave();

#ifdef BAD_LOCK
    // Acquire locks in the wrong order.
    printf("Acquiring lock at level 3\n");
    mutex1.Enter(); // Level 3

    printf("Acquiring bad lock at level 1!\n");
    // This line will cause the debugger to break!
    cs1.Enter();

    cs1.Leave();
    mutex1.Leave();
#endif
}
