// usermgr.c

#ifndef UNICODE
#define UNICODE
#endif

#include <windows.h> 
#include <lm.h>
#include "usermgr.h"
#include "account.h"
          
LRESULT CALLBACK MainDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK NewUserDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK NewGroupDlgProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK DelMemberDlgProc(HWND, UINT, WPARAM, LPARAM);
BOOL FillUsers(HWND); 
BOOL FillGroups(HWND); 
BOOL FillMembers(HWND, LPWSTR); 

HINSTANCE hInst;

//-----------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine, int nCmdShow)
{
    hInst = hInstance;

    DialogBox(hInst, MAKEINTRESOURCE(USER_MANAGER), NULL, 
              MainDlgProc);
    return 0;
}

//-----------------------------------------------------------------
LRESULT CALLBACK MainDlgProc(HWND hDlg, UINT msg, WPARAM wParam, 
                             LPARAM lParam)
{
    LONG Index;
    WCHAR UserName[LM20_UNLEN+1], GroupName[LM20_GNLEN+1];

    switch (msg) {
    case WM_INITDIALOG:
        FillUsers(hDlg);
        FillGroups(hDlg);
        break;

    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        case ADD_USER:        
            if (DialogBox(hInst,MAKEINTRESOURCE(NEW_USER_INFO), 
                          NULL, NewUserDlgProc)) {
                FillUsers(hDlg);
            }
            break;

        case DELETE_USER:        
            Index = SendDlgItemMessage(hDlg,USERS_LIST,LB_GETCURSEL,
                                       0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }

            SendDlgItemMessage(hDlg, USERS_LIST, LB_GETTEXT, Index,
                               (LPARAM)UserName); 
            if (DeleteUser(UserName, NULL)) {
                FillUsers(hDlg);
            }
            break;

        case ADD_GROUP:
            if (DialogBox(hInst,MAKEINTRESOURCE(NEW_GROUP_INFO), 
                          NULL, NewGroupDlgProc)) {
                FillGroups(hDlg);
            }
            break;

        case DELETE_GROUP:        
            Index = SendDlgItemMessage(hDlg,GROUPS_LIST,LB_GETCURSEL,
                                       0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }
            SendDlgItemMessage(hDlg, GROUPS_LIST, LB_GETTEXT, Index,
                               (LPARAM)GroupName); 
            if (DeleteGroup(GroupName, NULL)) {
                FillGroups(hDlg);
            }
            break;

        case ADD_USER_TO_GROUP:
            Index = SendDlgItemMessage(hDlg,USERS_LIST,LB_GETCURSEL,
                                       0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }
            SendDlgItemMessage(hDlg, USERS_LIST, LB_GETTEXT, Index,
                               (LPARAM)UserName); 
        
            Index = SendDlgItemMessage(hDlg,GROUPS_LIST,LB_GETCURSEL,
                                       0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }
            SendDlgItemMessage(hDlg, GROUPS_LIST, LB_GETTEXT, Index,
                               (LPARAM)GroupName); 
        
            AddUserToGroup(UserName, GroupName, NULL);
            break;

        case DELETE_USER_FROM_GROUP:
            Index = SendDlgItemMessage(hDlg,GROUPS_LIST,LB_GETCURSEL,
                                       0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }
            SendDlgItemMessage(hDlg, GROUPS_LIST, LB_GETTEXT, Index,
                               (LPARAM)GroupName); 
            
            DialogBoxParam(hInst,MAKEINTRESOURCE(DELETE_MEMBER), 
                           NULL, DelMemberDlgProc, 
                           (LPARAM)GroupName);
            break;
            
        case IDOK:
            EndDialog(hDlg, TRUE);
            return TRUE;
        }
        break;
    }
    return FALSE;
}

//-----------------------------------------------------------------
LRESULT CALLBACK NewUserDlgProc(HWND hDlg, UINT msg, WPARAM wParam, 
                                LPARAM lParam)
{
    BOOL Status;
    WCHAR UserName[LM20_UNLEN+1], Password[LM20_PWLEN+1];

    switch (msg) {
    case WM_INITDIALOG:        
        SendDlgItemMessage(hDlg, ID_USERNAME, EM_SETLIMITTEXT, 
                           LM20_UNLEN, 0);         
        SendDlgItemMessage(hDlg, ID_PASSWORD, EM_SETLIMITTEXT, 
                           LM20_PWLEN, 0); 
        break;

    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        
        case IDOK:            
            SendDlgItemMessage(hDlg, ID_USERNAME, WM_GETTEXT, 
                               LM20_UNLEN+1, (LPARAM)UserName);                     
            SendDlgItemMessage(hDlg, ID_PASSWORD, WM_GETTEXT, 
                               LM20_PWLEN+1, (LPARAM)Password);         
            Status = CreateUser(UserName, Password, NULL);
            EndDialog(hDlg, Status);
            return TRUE;
        
        case IDCANCEL:
            EndDialog(hDlg, FALSE);
            return TRUE;
        }
        break;
    }
    return FALSE;
}

//-----------------------------------------------------------------
LRESULT CALLBACK NewGroupDlgProc(HWND hDlg, UINT msg, WPARAM wParam, 
                                 LPARAM lParam)
{
    BOOL Status;
    WCHAR GroupName[LM20_GNLEN+1];

    switch (msg) {
    case WM_INITDIALOG:   
        SendDlgItemMessage(hDlg, ID_GROUPNAME, EM_SETLIMITTEXT, 
                           LM20_GNLEN, 0);         
        break;

    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        
        case IDOK:            
            SendDlgItemMessage(hDlg, ID_GROUPNAME, WM_GETTEXT, 
                               LM20_GNLEN+1, (LPARAM)GroupName);                     
            Status = CreateGroup(GroupName, NULL);
            EndDialog(hDlg, Status);
            return TRUE;
        
        case IDCANCEL:
            EndDialog(hDlg, FALSE);
            return TRUE;
        }
        break;
    }
    return FALSE;
}

//-----------------------------------------------------------------
LRESULT CALLBACK DelMemberDlgProc(HWND hDlg, UINT msg, 
                                  WPARAM wParam, LPARAM lParam)
{
    static LPWSTR GroupName;
    LONG Index;
    WCHAR MemberName[LM20_UNLEN+1];
    
    switch (msg) {
    case WM_INITDIALOG: 
        GroupName = (LPWSTR)lParam;
        SetDlgItemText(hDlg, IDS_LOCALGROUP, GroupName);
        FillMembers(hDlg, GroupName);
        break;

    case WM_COMMAND:
        switch(LOWORD(wParam)) {
        
        case IDOK:            
            Index = SendDlgItemMessage(hDlg, MEMBER_LIST,
                                       LB_GETCURSEL, 0, 0);
            if (Index == LB_ERR) {
                MessageBeep(0);
                break;
            }
            SendDlgItemMessage(hDlg, MEMBER_LIST, LB_GETTEXT, 
                               Index, (LPARAM)MemberName); 
            
            DeleteMemberFromGroup(MemberName, GroupName, NULL);
            FillMembers(hDlg, GroupName);
            return TRUE;
        
        case IDCANCEL:
            EndDialog(hDlg, TRUE);
            return TRUE;
        }
        break;
    }
    return FALSE;
}

//-----------------------------------------------------------------
BOOL FillUsers(HWND hDlg) 
{
    NET_API_STATUS NetStatus = NERR_Success;
    LPBYTE Data;
    DWORD Total = 0, i;
    NET_DISPLAY_USER *UserInfo;
    
    SendDlgItemMessage(hDlg, USERS_LIST, LB_RESETCONTENT, 0, 0);
    
    NetStatus = NetQueryDisplayInformation(NULL, 1, 0, 256, 4096,
                                           &Total, &Data);

    if (NetStatus != NERR_Success || Data == NULL) {
        return FALSE;
    }

    UserInfo = (NET_DISPLAY_USER *)Data;
    for (i=0; i < Total; i++) {
        SendDlgItemMessage(hDlg, USERS_LIST, LB_ADDSTRING, 0, 
            (LPARAM)(LPCTSTR)UserInfo->usri1_name);
        UserInfo++;
    }
    NetApiBufferFree(Data);
    return TRUE;
}

//-----------------------------------------------------------------
BOOL FillGroups(HWND hDlg)
{
    NET_API_STATUS NetStatus = NERR_Success;
    LPBYTE Data;
    DWORD Index = 0, ResumeHandle = 0, Total = 0, i;
    LOCALGROUP_INFO_0 *GroupInfo;

    SendDlgItemMessage(hDlg, GROUPS_LIST, LB_RESETCONTENT, 0, 0);
    
    NetStatus = NetLocalGroupEnum(NULL, 0, &Data, 4096, &Index, 
                                  &Total, &ResumeHandle);
    
    if (NetStatus != NERR_Success || Data == NULL) {
        return FALSE;
    }

    GroupInfo = (LOCALGROUP_INFO_0 *)Data;
    for (i=0; i < Total; i++) {
        SendDlgItemMessage(hDlg, GROUPS_LIST, LB_ADDSTRING, 0, 
            (LPARAM)(LPCTSTR)GroupInfo->lgrpi0_name);
        GroupInfo++;
    }
    NetApiBufferFree(Data);
    return TRUE;
}

//-----------------------------------------------------------------
BOOL FillMembers(HWND hDlg, LPWSTR GroupName) 
{
    NET_API_STATUS NetStatus = NERR_Success;
    LPBYTE Data;
    DWORD Index = 0, ResumeHandle = 0, Total = 0, i;
    LOCALGROUP_MEMBERS_INFO_1 *MemberInfo;

    SendDlgItemMessage(hDlg, MEMBER_LIST, LB_RESETCONTENT, 0, 0);
    
    NetStatus = NetLocalGroupGetMembers(NULL, GroupName, 1, &Data, 
                                        4096, &Index, &Total, 
                                        &ResumeHandle);
    
    if (NetStatus != NERR_Success || Data == NULL) {
        return FALSE;
    }

    MemberInfo = (LOCALGROUP_MEMBERS_INFO_1 *)Data;
    for (i=0; i < Total; i++) {
        SendDlgItemMessage(hDlg, MEMBER_LIST, LB_ADDSTRING, 0, 
            (LPARAM)(LPCTSTR)MemberInfo->lgrmi1_name);
        MemberInfo++;
    }
    NetApiBufferFree(Data);
    return TRUE;
}

